// This file is part of BUDEES, a software that allows fast simulation of bubble dynamics in electrolyzers and their interactions with electrical parameters. 
// 
// Copyright (C) 2024 <CNRS-ENS Rennes-Université de Rennes-CY Cergy Paris Université-Université Paris Saclay-CNAM Paris-ENS Paris-Saclay>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with this program. If not, see https://www.gnu.org/licenses/

#define ANODE 0
#define O2 0
#define CATHODE 1
#define H2 1


/* --------- Material --------- */
double dSite = 1.5/(1e-3); // Hole density [1/m]
double RFritz[2] = {60e-6, 60e-6}; // Rayon de Fritz [m]
double theta[2] = {M_PI, M_PI}; // Angle de mouillage des bulles (de l'ordre de PI) [rad]
/* -------- Electrolyte ------- */
double mu = 1e-3; // Viscosité dynamique [Pa.s]
double gravity = 9.81; // Gravity [m/s2]
double rho = 997; // Density [kg/m3]
double P0 = 1.01e5; // Pressure [Pa]
double T0 = 273.15 + 35.0; // Temperature [K]
double gammaLiquidGas[2] = {70e-3, 70e-3}; // Surface tensions [N/m]
/* ------ Electrochemistry ---- */
int Nelectronic[2] = {4, 2}; // Nombre d'électron nécessaire à la formation d'une molécule
double sigma = 2; // Electrolyte conductivity [S/m]
double tafelA0[2] = {(60e-3)/log(10), (30e-3)/log(10)}; // Tafel modeling : voltage slope [V]
double tafelJ0[2] = {1e-3, 1e-1}; // Tafel modeling : exchange current [A/m2]
/* ---------------------------- */




/* ------------ Coupling ---------- */
bool electricalCoupling = false; // Couplage faible du champ électrique
/* ------------------------------- */

/* ----------- Repeat y ---------- */
bool repeatYBubble = false; // Simule la répétition du motif selon y (recopie des bulles 1 fois)
/* ------------------------------- */

/* -------------- Mesh ------------ */
int meshN = 400; // Mesh size [-]
double BUBBLE_MESH_COEFF = 5.0; // Bubble meshing factor for bubble diameter [-]
/* -------------------------------- */

/* ------ Solver tolerance ----- */
#define TOLERANCE_LU 1e-10 // Tolérance sur la vérification de l'inverse obtenu via la méthode LU
#define TOLERANCE_POTENTIAL 1e-6 // Critère d'arrêt sur la résolution du potentiel électrolytique
#define TOLERANCE_CURRENT 1e-3 // Critère d'arrêt sur la résolution du courant
#define DERIVATIVE_VARIATION 1e-2 // Fraction de la grandeur pour l'approximation de la dérivée
#define SOLVER_MAX_ITERATION 40 // Nombre maximum d'itération (pour les cas non solvables)
/* ----------------------------- */

/*---------- Time step -------- */
double GROWING_TIME_COEF = 5.0; // Growing time step factor [-]
double BUOYANCY_TIME_COEF = 10.0; // Buoyancy time step factor [-]
double ELECTRICAL_TIME_COEF = 1.0; // Buoyancy time step factor [-]
/* ---------------------------- */



/* -- Numerical parameters -- */
/* -------------------------- */
double growingKineticFraction = 0.7; // Proportion de croissance/attente sur un site (croissance)
double detachementMinimalRadiusNorm = 0.8; // Fraction du rayon de détachement naturel à partir duquel le détachement dynamique est possible
/* -------------------------- */
#define interactionLongMaximumRange 15 // Portée maximale de l'interaction champ lointain (normalisée par le rayon moyen)
#define interactionLubMaximumRange 3.5 // Portée maximale de l'interaction lubrification (normalisée par le rayon moyen)
/*--------- Repulsive ------- */
double interactionRepulsiveRange = 2.2; // Portée du modèle de répulsion (Rref)
double interactionRepulsiveVelocity = 5e0; // Raideur du modèle de répulsion (Uref)
/* ----- Anti-collision ----- */
#define interactionCollisionRange 3 // Portée à partir de laquelle le contact est testé (normalisée par le rayon moyen)
#define interactionSafetyD 0.1 // Distance de sécurité à partir de laquelle la vitesse normale relative est absorbée
#define extraRFritz 0.02 // Rayon supplémentaire au rayon de Fritz pour éviter le blocage numérique
/* -------------------------- */
#define cloggedTimeFactor 1 // Temps à partir duquel une bulle est considérée coincée
/* -------------------------- */




/* ---------- Ouptut --------- */
/* --------------------------- */
bool outputElectrical = true;
string outputFolder = "output";
int outputMode = 0;
// Mode 0 : all frames
// Mode 1 : real time recording during the entire simulation
int outputFps = 60, outputTimeFactor = 1/10;
// Mode 2 : all frame between given interval
double outputStart = 0, outputStop = 1;
/* --------------------------- */
bool useSubProgressBar = true;
/* --------------------------- */

