// This file is part of BUDEES, a software that allows fast simulation of bubble dynamics in electrolyzers and their interactions with electrical parameters. 
// 
// Copyright (C) 2024 <CNRS-ENS Rennes-Université de Rennes-CY Cergy Paris Université-Université Paris Saclay-CNAM Paris-ENS Paris-Saclay>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with this program. If not, see https://www.gnu.org/licenses/

#define MAX(a,b) ((a)>(b) ? (a) : (b))
#define MIN(a,b) ((a)<(b) ? (a) : (b))
#define ABS(x) ((x) < 0.0 ? (-(x)) : (x))
#define SQ(x) ((x)*(x))
#define CUBE(x) ((x)*(x)*(x))

#define SMALLEST 5.0e-6
#define INF 100000.0
#define INF_NORMALIZED 100.0


/* ------ Basics ------ */
/* -------------------- */
double mathAbs(double x) {
	return (x < 0.0) ? -x : x;
}
double mathSQ(double x) {
	return x*x;
}
double mathCB(double x) {
	return x*x*x;
}
int mathSign(double x) {
	return (x == 0) ? 0 : ((x > 0) ? +1 : -1);
}
double mathMax(double a, double b) {
	return (a > b) ? a : b;
}
double mathMin(double a, double b) {
	return (a < b) ? a : b;
}
double mathMinMax(double x, double min, double max) {
	return (x < min) ? min : ((x > max) ? max : x);
}
double mathMap(double x, double xMin, double xMax, double yMin, double yMax) {
	return yMin + (x-xMin)*(yMax-yMin)/(xMax-xMin);
}
/* -------------------- */

/* ------ Random ------ */
/* -------------------- */
bool mathRandInitialized = false;
void mathRandInit() {
	if (!mathRandInitialized) {
		srand(0); // Constant seed for easy debug
		mathRandInitialized = true;
	}
}
double mathRand() {
	return ((double) rand() / (RAND_MAX));
}
/* -------------------- */


/* ---- Vectors and co ---- */
/* ------------------------ */
bool pointEquals(Point& P1, Point& P2) {
	return ((P1).x == (P2).x) && ((P1).y == (P2).y);
}
double vectLength(Vect& u) {
	return sqrt(mathSQ(u.x) + mathSQ(u.y));
}
double vectProject(Vect& u, Vect& v) {
	return u.x*v.x + u.y*v.y;
}
Point vectForward(Point& x0, Vect& u, double pos) {
	return {x0.x + (u.x*pos), x0.y + (u.y*pos)};
}
/* ------------------------ */



/* ---- Index <-> position ---- */
/* ---------------------------- */
void indexToPosition(int* N, int i, int j, double *x, double *y) {
	*x = ((double) i)/(N[0]-1);
	*y = ((double) j)/(N[1]-1);
}
void positionToIndex(int* N, int *i, int *j, double x, double y) {
	*i = round((N[0]-1)*x);
	*j = round((N[1]-1)*y);
}
int indexToK(int* N, int i, int j) {
	return i + (N[0]*j);
}
/* ---------------------------- */




/* --- Linear system solver --- */
/* ---------------------------- */
int solveLU(VectorXd& x, SpMat& A, VectorXd& b) {
	// Decompose LU
	Eigen::SparseLU<SpMat> solver;
	solver.compute(A);

	// Decomposition failed
	if(solver.info() != Eigen::Success) return 1;

	// Solve
	x = solver.solve(b);

	// Solving failed
	if(solver.info() != Eigen::Success) return 2;

	// Check tolerance
	double residual = (((A*x)-b).norm() / b.norm());
	if (residual > TOLERANCE_LU) return 3;

	return 0;
}
int invertLU(SpMat& Ainv, SpMat& A) {
	// Decompose LU
	Eigen::SparseLU<SpMat> solver;
	solver.compute(A);

	// Decomposition failed
	if(solver.info() != Eigen::Success) return 1;

	// Solve
	SpMat I = Eigen::MatrixXd::Identity(A.rows(), A.cols()).sparseView();
	Ainv = solver.solve(I);

	// Solving failed
	if(solver.info() != Eigen::Success) return 2;

	// Check tolerance
	double residual = (A*Ainv-I).norm() / I.norm();
	if (residual > TOLERANCE_LU) return 3;

	return 0;
}
/* ---------------------------- */




/* ---------- Others ---------- */
/* ---------------------------- */
double bounds(double x, double min, double max) {
	double range = max - min;
	while (x >= max)
		x -= range;
	while (x < min)
		x += range;
	return x;
}
int isApprox(double a, double b, double tol) {
	return ((a > b*(1.0-tol)) && (a < b*(1.0+tol)));
}
bool isBetween(int i, int iMin, int iMax) {
	return (i >= iMin) && (i <= iMax);
}
bool isBetween(int i, int j, int* N) {
	return isBetween(i, 0, N[0]-1) && isBetween(j, 0, N[1]-1);
}
/* ---------------------------- */
